function []= MotionDetectionCromatic(nome_cartella, estensione_file, esposizione, movimento, num_noise)

% MotionDetectionCromatic(nome_cartella, esposizione, movimento, num_noise)
% Rileva il movimento da due immagini a lunga esposizione
%
% le immagini statiche e di movimento devono essere nella stessa cartella
% di nome /nome_cartella/esposizione
% Le 2 immagini di movimento devono chiamarsi:
%                   - esposizione_movimento_1.estensione_file
%                   - esposizione_movimento_2.estensione_file
% Le n immagini statiche utilizzate per calcolare la soglia del rumore devono chiamarsi:
%                   - esposizione_noise_n.estensione_file
%   Esempio: per analizzare due immagini, in formato tif, di 5 secondi di esposizione, rappresentanti 
%            il primo movimento da analizzare in un giardino,utilizzando 10 immagini 
%            statiche per il calcolo della soglia esse dovranno essere poste in una 
%            cartella chiamata '5' posta in una cartella chiamata ad esempio 'Giardino'.
%            Nella cartella '5' dovranno essere presenti le due immagini di
%            movimento chiamate 5_1_1.tif e 5_1_2.tif, e le 10 immagini
%            statiche per calcolare la soglia del rumore chiamate
%            5_noise_1.tif, 5_noise_2.tif, ... , 5_noise_10.tif
%
% input:
%   - nome_cartella: directory nella quale sono contenuti le immagini a
%                    lunga esposizione
%   - estensione_file: estensione dei file di immagine
%   - esposizione: tempo di esposizione in secondi delle immagini da
%                  analizzare
%   - movimento: intero che indica il movimento utilizzato
%   - num_noise: numero delle immagini statiche di rumore da elaborare
%
% Il metodo utilizzato calcola le differenze cromatiche tra le due immagini.
% Per il calcolo della soglia di rumore viene calcolata la varianza di ogni
% pixel delle immagini statiche della scena da osservare. 
% Vengono utilizzate pi soglie di rumore, ottenute moltiplicando la varianza
% di ogni pixel per 1, 2, 3, 4, 6, in modo da classificare i livelli di
% movimento

%dimensione del filtro mediano, impostato in modo che sia grande circa quanto la
%testa di una persona nella scena

dim_testa = [15 15];

%% LETTURA IMMAGINI E RESIZE
    
    %Dimensioni immagini resizate
    width = 800;
    height = 533;
     
         %Leggo le N immagini statiche per rilevare la soglia di rumore
    %e le metto in questa matrice tridimensionale gi ridimensionale
    matrix_noise = zeros(height, width, num_noise*3); %per contenere tutte le immagini con i loro canali
    for i=1:num_noise
        im_noise = im2double(uint16((imread(strcat(nome_cartella,'/',esposizione,'/',esposizione,'_noise_',num2str(i),'.',estensione_file)))));
        im_noise_resized=imresize(im_noise,[height width]);
        
        matrix_noise(:,:,1+3*(i-1)) = im_noise_resized(:,:,1);%salvo il canale rosso
        matrix_noise(:,:,2+3*(i-1)) = im_noise_resized(:,:,2);%salvo il canale verde
        matrix_noise(:,:,3+3*(i-1)) = im_noise_resized(:,:,3);%salvo il canale blu
    end
    
    %Leggo le 2 immagini da confrontare
    
    im1 = im2double((imread(strcat(nome_cartella,'/',esposizione,'/',esposizione,'_',movimento,'_1.',estensione_file))));
    im1 = imresize(im1,[height width]);
    im2 = im2double((imread(strcat(nome_cartella,'/',esposizione,'/',esposizione,'_',movimento,'_2.',estensione_file))));
    im2 = imresize(im2,[height width]);

    
    %conversione delle immagini in HSV
    im1hsv=rgb2hsv(im1);
    im2hsv=rgb2hsv(im2);
    im1h=im1hsv(:,:,1);
    im1s=im1hsv(:,:,2);
    im2h=im2hsv(:,:,1);
    im2s=im2hsv(:,:,2);
    
    
%% CALCOLO DELLE DIFFERENZE CROMATICHE NELLE IMMAGINI DEL MOVIMENTO

    %calcolo la differenza
    I = zeros(height,width);
 
    for x=1:height 
        for y=1:width 

            %differenza nell'immagine

            
            I(x,y)=norm(im1s(x,y).*[cos(im1h(x,y)*2*pi) sin(im1h(x,y)*2*pi)] - im2s(x,y).*[cos(im2h(x,y)*2*pi) sin(im2h(x,y)*2*pi)]);
            

        end
    end
    
%% APPLICAZIONE DEL FILTRO MEDIANO

    %applico il filtro mediano anche a tutte le immagini di rumore
    for i=1:num_noise
        matrix_noise(:,:,1+3*(i-1)) = medfilt2( matrix_noise(:,:,1+3*(i-1)),dim_testa);
        matrix_noise(:,:,2+3*(i-1)) = medfilt2( matrix_noise(:,:,2+3*(i-1)),dim_testa);
        matrix_noise(:,:,3+3*(i-1)) = medfilt2( matrix_noise(:,:,3+3*(i-1)),dim_testa);
    end


    %% RICERCA DELLA SOGLIA (METODO SEGNALE + RUMORE GAUSSIANO / VARIANZA)

    %per ogni pixel, per ogni canale, calcolo la media sulle N immagini
    valore_medio_canali = zeros(height, width, 3);
    for i=1:height 
        for j=1:width 
              valore_rosso = 0;
              valore_verde = 0;
              valore_blu = 0;
              for im=1:num_noise
                valore_rosso = valore_rosso + matrix_noise(i,j,1+3*(im-1));
                valore_verde = valore_verde + matrix_noise(i,j,2+3*(im-1));
                valore_blu   = valore_blu + matrix_noise(i,j,3+3*(im-1));
              end
              media_rosso = valore_rosso/num_noise;
              media_verde = valore_verde/num_noise;
              media_blu = valore_blu/num_noise;
              
              valore_medio_canali(i, j, 1) = media_rosso;
              valore_medio_canali(i, j, 2) = media_verde;
              valore_medio_canali(i, j, 3) = media_blu;
        end
    end
    
    %Per ogni pixel, per ogni canale, calcolo la varianza
    varianza_canali = zeros(height, width, 3);
    for i=1:height 
        for j=1:width 
              valore_rosso = 0;
              valore_verde = 0;
              valore_blu = 0;
              media_rosso = valore_medio_canali(i, j, 1);
              media_verde =valore_medio_canali(i, j, 2);
              media_blu = valore_medio_canali(i, j, 3);
              for im=1:num_noise
                valore_rosso = valore_rosso + (matrix_noise(i,j,1+3*(im-1))-media_rosso)^2;
                valore_verde = valore_verde + (matrix_noise(i,j,2+3*(im-1))-media_verde)^2;
                valore_blu   = valore_blu + (matrix_noise(i,j,3+3*(im-1))-media_blu)^2;
              end
              varianza_rosso = valore_rosso/num_noise;
              varianza_verde = valore_verde/num_noise;
              varianza_blu = valore_blu/num_noise;
              
              varianza_canali(i, j, 1) = varianza_rosso;
              varianza_canali(i, j, 2) = varianza_verde;
              varianza_canali(i, j, 3) = varianza_blu;
        end
    end
    
    %Ricavo la deviazione standard per ogni pixel
    deviazione_standard = sqrt(varianza_canali);
    
    deviazione_rosso = mean(mean(mean(deviazione_standard(:,:,1))));
    deviazione_verde = mean(mean(mean(deviazione_standard(:,:,2))));
    deviazione_blu = mean(mean(mean(deviazione_standard(:,:,3))));
    
    deviazione_media= (deviazione_rosso + deviazione_verde + deviazione_blu)/3;

%% APPLICAZIONE DELLA SOGLIA

    %creo una immagine RGB vuota nera
    motion = zeros(height, width); 
        
    for i=1:height 
        for j=1:width 


            if(I(i,j) > deviazione_media*6)
                motion(i,j) = 1;
            elseif(I(i,j) > deviazione_media*4.5)
                motion(i,j) = 0.6;
            elseif(I(i,j) > deviazione_media*3.5)
                motion(i,j) = 0.4;
            elseif(I(i,j) > deviazione_media*2.5)
                motion(i,j) = 0.2;
            elseif(I(i,j) > deviazione_media*1.5)
                motion(i,j) = 0.05;
            end

        end
    end
    
    
%% SALVATAGGIO DELLE IMMAGINI

% vengono salvate 2 immagini: 
%   Differenze cromatiche
    imwrite (I, strcat([nome_cartella,'/',esposizione,'/',nome_cartella,' ',esposizione,' sec - ',num2str(movimento),'Differenza Cromatica.jpeg']), 'Quality', 100);
%   Movimento rilevato
    imwrite (motion, strcat([nome_cartella,'/',esposizione,'/',nome_cartella,' ',esposizione,' sec - ',num2str(movimento),'Movimento Cromatico.jpeg']), 'Quality', 100);
    